var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
import React, { forwardRef } from 'react';
import posed from '../posed';
import PoseGroup from '../components/Transition/PoseGroup';
import { render, cleanup } from 'react-testing-library';
afterEach(cleanup);
var Parent = posed.div({
    init: { x: 10, transition: { duration: 30 } },
    foo: { x: 20, transition: { duration: 30 } },
    bar: { x: 30, transition: { duration: 30 } },
    fromProps: { x: function (_a) {
            var i = _a.i;
            return i;
        }, transition: { duration: 30 } },
    preEnter: { x: 40, transition: { duration: 30 } },
    enter: { x: 50, transition: { duration: 30 } },
    exit: { x: 60, transition: { duration: 30 } },
    dynamicEnter: { x: function (_a) {
            var x = _a.x;
            return x * 2;
        }, transition: { duration: 30 } },
    dynamicExit: {
        x: function (_a) {
            var x = _a.x;
            return x;
        },
        transition: { duration: 30 }
    }
});
var Child = posed.div({
    init: { y: 15, transition: { duration: 30 } },
    foo: { y: 25, transition: { duration: 30 } },
    bar: { y: 35, transition: { duration: 30 } },
    fromProps: { y: function (_a) {
            var i = _a.i;
            return i;
        }, transition: { duration: 30 } },
    preEnter: { y: 45, transition: { duration: 30 } },
    enter: { y: 55, transition: { duration: 30 } },
    exit: { y: 65, transition: { duration: 30 } },
    dynamicEnter: { y: 75, transition: { duration: 30 } },
    dynamicExit: { y: 85, transition: { duration: 30 } },
    dynamicExitDuration: {
        y: 85,
        transition: function (_a) {
            var i = _a.i;
            return ({ duration: (i + 1) * 30 });
        }
    }
});
test('posed: initial state - named initial pose', function () {
    var x = 0;
    var y = 0;
    render(React.createElement(Parent, { pose: "foo", onValueChange: { x: function (v) { return (x = v); } } },
        React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })));
    expect(x).toBe(20);
    expect(y).toBe(25);
});
test('posed: initial state - auto `init` pose', function () {
    var x = 0;
    var y = 0;
    render(React.createElement(Parent, { onValueChange: { x: function (v) { return (x = v); } } },
        React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })));
    expect(x).toBe(10);
    expect(y).toBe(15);
});
test('posed: mount animation with `initialPose`', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        render(React.createElement(Parent, { initialPose: "bar", pose: "foo", onPoseComplete: function () {
                expect(x).toBe(20);
                expect(y).toBe(25);
                resolve();
            }, onValueChange: { x: function (v) { return (x = v); } } },
            React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })));
        expect(x).toBe(30);
        expect(y).toBe(35);
    });
});
test('posed: passes through props', function () {
    var x = 0;
    var y = 0;
    render(React.createElement(Parent, { pose: "fromProps", i: 1, onValueChange: { x: function (v) { return (x = v); } } },
        React.createElement(Child, { i: 2, onValueChange: { y: function (v) { return (y = v); } } })));
    expect(x).toBe(1);
    expect(y).toBe(2);
});
test('posed: `onPoseComplete` gets called with pose as argument', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        render(React.createElement(Parent, { initialPose: "bar", pose: "foo", onPoseComplete: function (pose) {
                expect(pose).toBe('foo');
                resolve();
            } }));
    });
});
test('PoseGroup: Initial visibility (visible)', function () {
    var x = 0;
    var y = 0;
    var Group = function (_a) {
        var _b = _a.isVisible, isVisible = _b === void 0 ? true : _b;
        return (React.createElement(PoseGroup, null, isVisible && (React.createElement(Parent, { key: "a", onValueChange: { x: function (v) { return (x = v); } } },
            React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
    };
    render(React.createElement(Group, null));
    expect(x).toBe(50);
    expect(y).toBe(55);
});
test('PoseGroup: Initial visibility (hidden)', function () {
    var x = 0;
    var y = 0;
    var Group = function (_a) {
        var _b = _a.isVisible, isVisible = _b === void 0 ? false : _b;
        return (React.createElement(PoseGroup, null, isVisible && (React.createElement(Parent, { key: "a", onValueChange: { x: function (v) { return (x = v); } } },
            React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
    };
    render(React.createElement(Group, null));
    expect(x).toBe(0);
    expect(y).toBe(0);
});
test('PoseGroup: Animate on mount', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? true : _b;
            return (React.createElement(PoseGroup, { animateOnMount: true }, isVisible && (React.createElement(Parent, { key: "a", onPoseComplete: function () {
                    expect(x).toBe(50);
                    expect(y).toBe(55);
                    resolve();
                }, onValueChange: { x: function (v) { return (x = v); } } },
                React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
        };
        render(React.createElement(Group, null));
        expect(x).toBe(60);
        expect(y).toBe(65);
    });
});
test('PoseGroup: onRest fires', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? true : _b;
            return (React.createElement(PoseGroup, { onRest: function () {
                    expect(x).toBe(60);
                    expect(y).toBe(65);
                    resolve();
                } }, isVisible && (React.createElement(Parent, { key: "a", onValueChange: { x: function (v) { return (x = v); } } },
                React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
        };
        var rerender = render(React.createElement(Group, null)).rerender;
        rerender(React.createElement(Group, { isVisible: false }));
        expect(x).toBe(50);
        expect(y).toBe(55);
    });
});
test('PoseGroup: onRest fires when exit pose starts during exit pose', function () {
    var range = function (n) { return Array.from({ length: n }, function (_, i) { return i; }); };
    return new Promise(function (resolve) {
        var Group = (function (_super) {
            __extends(Group, _super);
            function Group() {
                var _this = _super !== null && _super.apply(this, arguments) || this;
                _this.state = {
                    list: range(6)
                };
                _this.pop2 = function () {
                    var list = _this.state.list;
                    if (!list.length) {
                        return;
                    }
                    _this.setState({ list: list.slice(0, -2) });
                };
                return _this;
            }
            Group.prototype.componentDidMount = function () {
                this.pop2();
            };
            Group.prototype.render = function () {
                var _this = this;
                return (React.createElement(PoseGroup, { exitPose: "dynamicExitDuration", onRest: resolve }, this.state.list.map(function (i) { return (React.createElement(Child, { i: i, key: i, onPoseComplete: _this.pop2 })); })));
            };
            return Group;
        }(React.Component));
        var wrapper = render(React.createElement(Group, null));
    });
});
test('PoseGroup: Animate conditionally', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? false : _b;
            return (React.createElement(PoseGroup, { animateOnMount: true }, isVisible && (React.createElement(Parent, { key: "a", onPoseComplete: function () {
                    expect(x).toBe(50);
                    expect(y).toBe(55);
                    resolve();
                }, onValueChange: { x: function (v) { return (x = v); } } },
                React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
        };
        var rerender = render(React.createElement(Group, null)).rerender;
        expect(x).toBe(0);
        expect(y).toBe(0);
        rerender(React.createElement(Group, { isVisible: true }));
        expect(x).toBe(60);
        expect(y).toBe(65);
    });
});
test('PoseGroup: Forward props from PoseGroup to direct child', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? false : _b;
            return (React.createElement(PoseGroup, { animateOnMount: true, enterPose: "dynamicEnter", exitPose: "dynamicExit", preEnterPose: "dynamicExit", x: 100 }, isVisible && (React.createElement(Parent, { key: "a", onPoseComplete: function () {
                    expect(x).toBe(200);
                    expect(y).toBe(75);
                    resolve();
                }, onValueChange: { x: function (v) { return (x = v); } } },
                React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
        };
        var rerender = render(React.createElement(Group, null)).rerender;
        expect(x).toBe(0);
        expect(y).toBe(0);
        rerender(React.createElement(Group, { isVisible: true }));
        expect(x).toBe(100);
        expect(y).toBe(85);
    });
});
test('PoseGroup: Override props on child', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? false : _b;
            return (React.createElement(PoseGroup, { animateOnMount: true, enterPose: "dynamicEnter", exitPose: "dynamicExit", preEnterPose: "dynamicExit", x: isVisible ? 101 : 333 }, isVisible && (React.createElement(Parent, { key: "a", onPoseComplete: function (pose) {
                    if (pose === 'dynamicExit') {
                        expect(x).toBe(333);
                        expect(y).toBe(85);
                        resolve();
                    }
                    else {
                        expect(x).toBe(202);
                        expect(y).toBe(75);
                        rerender(React.createElement(Group, { isVisible: false }));
                    }
                }, onValueChange: { x: function (v) { return (x = v); } } },
                React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
        };
        var rerender = render(React.createElement(Group, null)).rerender;
        expect(x).toBe(0);
        expect(y).toBe(0);
        rerender(React.createElement(Group, { isVisible: true }));
        expect(x).toBe(101);
        expect(y).toBe(85);
    });
});
test('PoseGroup: Provides group props to children on mount', function () {
    var x = 0;
    var y = 0;
    var ChildWithGroupProps = posed(forwardRef(function (_a, innerRef) {
        var text = _a.text, props = __rest(_a, ["text"]);
        return (React.createElement("div", __assign({ ref: innerRef }, props), "text: " + text));
    }))({});
    var Group = function () { return (React.createElement(PoseGroup, { animateOnMount: true, text: "foo bar" },
        React.createElement(ChildWithGroupProps, { "data-testid": "child", key: "a" }))); };
    var _a = render(React.createElement(Group, null)), getByTestId = _a.getByTestId, debug = _a.debug;
    var child = getByTestId('child');
    expect(child.textContent).toBe('text: foo bar');
});
test('PoseGroup: `onPoseComplete` gets called for leaving child', function () {
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? true : _b;
            return (React.createElement(PoseGroup, { animateOnMount: true }, isVisible && (React.createElement(Parent, { key: "a", onPoseComplete: function (pose) {
                    if (pose === 'enter') {
                        rerender(React.createElement(Group, { isVisible: false }));
                        return;
                    }
                    expect(pose).toBe('exit');
                    resolve();
                } },
                React.createElement(Child, null)))));
        };
        var rerender = render(React.createElement(Group, null)).rerender;
    });
});
test('PoseGroup: special pre-enter pose', function () {
    var x = 0;
    var y = 0;
    return new Promise(function (resolve) {
        var Group = function (_a) {
            var _b = _a.isVisible, isVisible = _b === void 0 ? false : _b;
            return (React.createElement(PoseGroup, { animateOnMount: true, preEnterPose: "foo" }, isVisible && (React.createElement(Parent, { key: "a", onPoseComplete: function () {
                    expect(x).toBe(50);
                    expect(y).toBe(55);
                    resolve();
                }, onValueChange: { x: function (v) { return (x = v); } } },
                React.createElement(Child, { onValueChange: { y: function (v) { return (y = v); } } })))));
        };
        var rerender = render(React.createElement(Group, null)).rerender;
        expect(x).toBe(0);
        expect(y).toBe(0);
        rerender(React.createElement(Group, { isVisible: true }));
        expect(x).toBe(20);
        expect(y).toBe(25);
    });
});
test('StrictMode: PoseGroup removes children correctly', function () {
    return new Promise(function (resolve) {
        var First = posed.div({
            enter: { opacity: 1 },
            exit: { opacity: 0 }
        });
        var Second = posed.div({
            enter: { opacity: 1 },
            exit: { opacity: 0 }
        });
        var Group = function (props) { return (React.createElement(React.StrictMode, null,
            React.createElement(PoseGroup, __assign({}, props)))); };
        var rerender = render(React.createElement(Group, null,
            React.createElement(First, { key: "first", onPoseComplete: function (pose) {
                    expect(pose).toBe('exit');
                    resolve();
                } }))).rerender;
        rerender(React.createElement(Group, null,
            React.createElement(Second, { key: "second" })));
    });
});
//# sourceMappingURL=index.test.js.map